#   OpenProximity2.0 is a proximity marketing OpenSource system.        
#   Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#                                                                          
#   This program is free software; you can redistribute it and/or modify  
#   it under the terms of the GNU General Public License as published by  
#   the Free Software Foundation version 2 of the License.                
#                                                                          
#   This program is distributed in the hope that it will be useful,       
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
#   GNU General Public License for more details.                          
#                                                                          
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.   

'''
A few useful constants.

Note: We should move return codes here!
'''

BLUEZ='org.bluez'
BLUEZ_PATH='/'
BLUEZ_ADAPTER='org.bluez.Adapter'
BLUEZ_MANAGER='org.bluez.Manager'

AIRCABLE_MAC=['00:50:C2', '00:25:BF']

UUID={
    'spp': '00001101-0000-1000-8000-00805f9b34fb',
    'opp': '00001105-0000-1000-8000-00805f9b34fb',
    'ftp': '00001106-0000-1000-8000-00805f9b34fb',
}

